#ifndef _pg_report_h_
#define _pg_report_h_

#define codeUnalloc           1
#define codeNoRouteSpec       2
#define codeNoRouteDest       3
#define codeChUnacep          6
#define codeCallRej           21
#define codeDestOutOrder      27
#define codeInvNumberFormat   28
#define codeNoCircChAv        34
#define codeNetworkOutOrder   38
#define codeTempFailure       41
#define codeReqNoCircChAv     44
#define codeResourcesUnav     47
#define codeOutCallsBarred    52
#define codeServOptUnav       63
#define codeNoBearerCapImp    65
#define codeIncompDest        88
#define codeRecOnTimerExp     102
#define codeProtError         111
#define codeInterworkUnspec   127
#define codeUserBusy          17
#define codeNoUserResp        18
#define codeNoRespFromUser    19

#define FLAG_OK		          0
#define FLAG_ERROR            1

#define MAX_NUM_EVENTS        40

#define LEFT_MARGIN           20
#define TOP_MARGIN            20
#define BOTTOM_MARGIN         44
#define DIMA4_HEIGHT          820
#define DIMA4_WIDTH           595
#define LINE_SPACING          17

#define MIN(x, y) ((x)<(y) ? (x):(y))
#define MAX(x, y) ((x)>(y) ? (x):(y))

#define START_EV_TAG          "\r\n        <ev"
#define START_EV_TAG_LEN      strlen(START_EV_TAG)

#define CALLER_STR               " caller=\""
#define EP1_NUMBER_STR           " ep1_number=\""
#define EP1_NAME_STR             " ep1_name=\""
#define EP1_DN_STR               " ep1_dn=\""
#define EP2_NUMBER_STR           " ep2_number=\""
#define EP2_NAME_STR             " ep2_name=\""
#define EP2_DN_STR               " ep2_dn=\""
#define STATUS_STR               " status=\""
#define TYPE_STR                 " type=\""
#define ENTRY_STR                " entry=\""

#define CALLER_LEN           strlen(CALLER_STR)
#define EP1_NUMBER_LEN       strlen(EP1_NUMBER_STR)
#define EP1_NAME_LEN         strlen(EP1_NAME_STR)
#define EP1_DN_LEN           strlen(EP1_DN_STR)
#define EP2_NUMBER_LEN       strlen(EP2_NUMBER_STR)
#define EP2_NAME_LEN         strlen(EP2_NAME_STR)
#define EP2_DN_LEN           strlen(EP2_DN_STR)
#define STATUS_LEN           strlen(STATUS_STR)
#define TYPE_LEN             strlen(TYPE_STR)
#define ENTRY_LEN            strlen(ENTRY_STR)

typedef struct _remote_ {
   char *remote_e164;
   char *remote_h323;
   char *remote_dn;
   char *remote;
} remote_t;

typedef struct _csv_infos_ {
   char *object_info;
   char *object_called;
   char *diverting;
   char *transferring;
} csv_infos_t;

extern const char* xml_events_print[];
extern int num_xml_events_print;
extern const char* pdf_events_attrib[];
extern int num_pdf_events_attrib;
extern const char* xml_events_attrib[];
extern int num_xml_events_attrib;
extern const char* call_flow_indexes_pdf[];
extern int num_call_flow_indexes_pdf;
extern const char* call_flow_indexes_xml[];
extern int num_call_flow_indexes_xml;
extern const char* msg_lookup_table[];
extern int num_msg_lookup_table;
extern const char* image_lookup_table[];
extern int num_image_lookup_table;
extern const char* column_lookup_table[];
extern int num_column_lookup_table;
extern const char* xml_lookup_table_open[];
extern const char* xml_lookup_table_close[];
extern const char* xml_lookup_table[];

enum {
    CALLER_PDF,
    CALLER_PDF_IMG,
    EP1_PDF_NUMBER,
    EP1_PDF_NAME,
    EP1_PDF_DN,
    EP1_PDF,
    EP1_PDF_IMG,
    EP2_PDF_NUMBER,
    EP2_PDF_NAME,
    EP2_PDF_DN,
    EP2_PDF,
    EP2_PDF_IMG,
    STATUS_PDF,
    STATUS_PDF_IMG,
    TYPE_PDF,
    ENTRY_PDF
};

enum {
    CALLER_XML,
    EP1_XML_NUMBER,
    EP1_XML_NAME,
    EP1_XML_DN,
    EP2_XML_NUMBER,
    EP2_XML_NAME,
    EP2_XML_DN,
    STATUS_XML,
    TYPE_XML,
    ENTRY_XML
};

enum {
    NUMBER_PDF,
    NAME_PDF,
    DN_PDF,
    FORMATTED_PDF,
};

enum {
    NUMBER_XML,
    NAME_XML,
    DN_XML,
};

enum {
    ALERT_TO_MSG,          /* "alert-to" */
    ALERT_FROM_MSG,        /* "alert-from" */
    SETUP_TO_MSG,          /* "setup-to" */
    SETUP_FROM_MSG,        /* "setup-from" */
    CONN_TO_MSG,           /* "conn-to" */
    CONN_FROM_MSG,         /* "conn-from" */
    TRANSFER_TO_MSG,       /* "transfer-to" */
    TRANSFER_FROM_MSG,     /* "transfer-from" */
    TRANSFER_REMOTE,       /* "transfer-remote" */
    TRANSFERRED_MSG,       /* "transfered" */
    FORWARDED_MSG,         /* "forwarded" */
    CONNECTED_MSG,         /* "connected" */
    DISC_FROM_MSG,         /* "disc-from" */
    DISC_TO_MSG,           /* "disc-to" */
    REL_FROM_MSG,          /* "rel-from" */
    REL_TO_MSG,            /* "rel-to" */
    CF_FROM_MSG,           /* "cf-from" */
    CF_TO_MSG,             /* "cf-to" */
    SCF_MSG,               /* "scf" */
    SCT_MSG,               /* "sct" */
    FROM_MSG,              /* "from" */
    TO_MSG,                /* "to" */
    CT_MSG,                /* "ct" */
    CF_MSG,                /* "cf" */
    EP_MSG,                /* "ep" */
    PICKUP_FROM_MSG,       /* "pickup-from" */
    REJECT_MSG,            /* "reject-from" */
    INFO_FROM_MSG,         /* "info-from" */
    INFO_TO_MSG,           /* "info-to" */
    UNKNOWN_MSG,           /* unknown */
};

enum {
    DUMMY = 0,
    BUSY,
    BUSY_CALLBACK,
    CONN,
    CONN_CALLBACK,
    DIR_IN,
    DIR_OUT,
    DIR_IN_PDF,
    DIR_OUT_PDF,
    ERROR,
    ERROR_CALLBACK,
    ERROR_PDF,
    CF,
    CF_REV,
    CF_PDF,
    CF_REV_PDF,
    CT,
    CT_REV,
    CT_PDF,
    CT_REV_PDF,
    CCT,
    CCT_REV,
    NO_RES,
    NO_RES_CALLBACK,
    NO_RES_PDF,
    STATE,
    PICKUP,
    PICKUP_REV,
    PICKUP_PDF,
    PICKUP_REV_PDF,
};

extern const char* image_load_table[];

#define NUM_PDF_IMAGES   sizeof(image_load_table)/sizeof(const char*)

enum {
    NO_ERROR,
    ERROR_CONN_DB,
    ERROR_MEM_CONN_DB,
    ERROR_QUERY,
    ERROR_MEM_QUERY,
    ERROR_HREF,
    ERROR_MEM_HREF,
    ERROR_FORMAT,
    ERROR_MEM_FORMAT,
    ERROR_SERVER,
    ERROR_MEM_SERVER,
    ERROR_FILENAME,
    ERROR_MEM_FILENAME,
    ERROR_MEM_SORDER,
    ERROR_MEM_SVALUE,
    ERROR_MEM_GVALUE,
    ERROR_MEM_LOGO,
    ERROR_MEM_FOOTER,
    ERROR_MEM_COLUMNS,
    ERROR_MEM_TRANSLATION,
    ERROR_CN,
    ERROR_MEM_CN,
    ERROR_MEM_LANGUAGE,
    ERROR_MEM_DISPLAYED_DURATION,
    ERROR_USER_GROUP,
    ERROR_MEM_USER_GROUP,
    ERROR_MEM_USER_PBX
};

typedef struct _xml_strings {
    char *tmpDir;
    char *state;
    char *type;
    char *true;
    char *empty;
    char *ct;
    char *cct;
    char *cf;
    char *co;
    char *al;
    char *er;
    char *bu;
    char *pu;
    char *ep1;
    char *ep2;
} xml_strings;

enum {
    TIME_COL,
    UNIX_TIME_COL,
    CN_COL,
    CONF_COL,
    DIR_COL,
    CALL_COL,
    ALERT_COL,
    CALLBACK_E164,
    CALLBACK_H323,
    CALLBACK_DN,
    CALLBACK_TIME,
    CALLBACK_TIME_UNIX,
    CALL_FLOW_COL,
    CALL_DUR_COL,
    CALL_CONN_COL,
    CALL_BILLING_COL,
    GROUPS_COL,
    SYSTEM_COL,
    NODE_COL,
    PBX_COL,
    PHYS_COL,
    CAUSE_COL,
    UTC_UNIX_TIME_COL,
    DATUM_COL,
    TIME_SHORT_COL,
    DATUM_SORT_COL,
    CDR_ID_COL,
    RECORDER_PATH_COL
};

int create_report(char *query_string, FCGX_Request *cgi, char * doc_root, int *bytes_sent, int *output_len);

#endif
